@@ -1,11 +1,9 @@
-from __future__ import annotations
-
-import asyncio
-from asyncio import CancelledError
+from concurrent.futures import CancelledError
 from itertools import chain
 from typing import TYPE_CHECKING, Any, Final, List, Optional, Union, cast
 
-from robotcode.core.async_tools import async_tasking_event, threaded
+from robotcode.core.concurrent import check_current_task_canceled
+from robotcode.core.event import event
 from robotcode.core.lsp.types import (
     CompletionContext,
     CompletionItem,
@@ -29,28 +27,32 @@ from robotcode.language_server.common.decorators import (
     HasTriggerCharacters,
     language_id_filter,
 )
-from robotcode.language_server.common.has_extend_capabilities import HasExtendCapabilities
-from robotcode.language_server.common.parts.protocol_part import LanguageServerProtocolPart
+from robotcode.language_server.common.parts.protocol_part import (
+    LanguageServerProtocolPart,
+)
 from robotcode.language_server.common.text_document import TextDocument
 
 if TYPE_CHECKING:
     from robotcode.language_server.common.protocol import LanguageServerProtocol
 
 
-class CompletionProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
+class CompletionProtocolPart(LanguageServerProtocolPart):
     _logger: Final = LoggingDescriptor()
 
-    def __init__(self, parent: LanguageServerProtocol) -> None:
+    def __init__(self, parent: "LanguageServerProtocol") -> None:
         super().__init__(parent)
 
-    @async_tasking_event
-    async def collect(
-        sender, document: TextDocument, position: Position, context: Optional[CompletionContext]  # NOSONAR
+    @event
+    def collect(
+        sender,
+        document: TextDocument,
+        position: Position,
+        context: Optional[CompletionContext],
     ) -> Union[List[CompletionItem], CompletionList, None]:
         ...
 
-    @async_tasking_event
-    async def resolve(sender, completion_item: CompletionItem) -> Optional[CompletionItem]:  # NOSONAR
+    @event
+    def resolve(sender, completion_item: CompletionItem) -> Optional[CompletionItem]:
         ...
 
     def extend_capabilities(self, capabilities: ServerCapabilities) -> None:
@@ -82,9 +84,8 @@ class CompletionProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
                 completion_item=CompletionOptionsCompletionItemType(label_details_support=True),
             )
 
-    @rpc_method(name="textDocument/completion", param_type=CompletionParams)
-    @threaded()
-    async def _text_document_completion(
+    @rpc_method(name="textDocument/completion", param_type=CompletionParams, threaded=True)
+    def _text_document_completion(
         self,
         text_document: TextDocumentIdentifier,
         position: Position,
@@ -95,7 +96,7 @@ class CompletionProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
         results: List[Union[List[CompletionItem], CompletionList]] = []
 
         if context is not None and context.trigger_kind == CompletionTriggerKind.TRIGGER_CHARACTER:
-            await asyncio.sleep(0.25)
+            check_current_task_canceled(0.25)
 
         document = self.parent.documents.get(text_document.uri)
         if document is None:
@@ -103,13 +104,15 @@ class CompletionProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
 
         p = document.position_from_utf16(position)
 
-        for result in await self.collect(
+        for result in self.collect(
             self,
             document,
             p,
             context,
             callback_filter=language_id_filter(document),
         ):
+            check_current_task_canceled()
+
             if isinstance(result, BaseException):
                 if not isinstance(result, CancelledError):
                     self._logger.exception(result, exc_info=result)
@@ -121,6 +124,8 @@ class CompletionProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
             return None
 
         for result in results:
+            check_current_task_canceled()
+
             if isinstance(result, CompletionList):
                 for item in result.items:
                     if item.text_edit is not None:
@@ -153,17 +158,11 @@ class CompletionProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
             item.text_edit.insert = document.range_to_utf16(item.text_edit.insert)
             item.text_edit.replace = document.range_to_utf16(item.text_edit.replace)
 
-    @rpc_method(name="completionItem/resolve", param_type=CompletionItem)
-    @threaded()
-    async def _completion_item_resolve(
-        self,
-        params: CompletionItem,
-        *args: Any,
-        **kwargs: Any,
-    ) -> CompletionItem:
+    @rpc_method(name="completionItem/resolve", param_type=CompletionItem, threaded=True)
+    def _completion_item_resolve(self, params: CompletionItem, *args: Any, **kwargs: Any) -> CompletionItem:
         results: List[CompletionItem] = []
 
-        for result in await self.resolve(self, params):
+        for result in self.resolve(self, params):
             if isinstance(result, BaseException):
                 if not isinstance(result, CancelledError):
                     self._logger.exception(result, exc_info=result)
