@@ -1,10 +1,8 @@
-from __future__ import annotations
-
-from asyncio import CancelledError
+import threading
+from concurrent.futures import CancelledError
 from typing import TYPE_CHECKING, Any, Final, List, Optional
 
-from robotcode.core.async_tools import async_tasking_event, threaded
-from robotcode.core.logging import LoggingDescriptor
+from robotcode.core.event import event
 from robotcode.core.lsp.types import (
     InlayHint,
     InlayHintOptions,
@@ -13,9 +11,9 @@ from robotcode.core.lsp.types import (
     ServerCapabilities,
     TextDocumentIdentifier,
 )
+from robotcode.core.utils.logging import LoggingDescriptor
 from robotcode.jsonrpc2.protocol import rpc_method
 from robotcode.language_server.common.decorators import language_id_filter
-from robotcode.language_server.common.has_extend_capabilities import HasExtendCapabilities
 from robotcode.language_server.common.text_document import TextDocument
 
 if TYPE_CHECKING:
@@ -24,18 +22,20 @@ if TYPE_CHECKING:
 from .protocol_part import LanguageServerProtocolPart
 
 
-class InlayHintProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
+class InlayHintProtocolPart(LanguageServerProtocolPart):
     _logger: Final = LoggingDescriptor()
 
-    def __init__(self, parent: LanguageServerProtocol) -> None:
+    def __init__(self, parent: "LanguageServerProtocol") -> None:
         super().__init__(parent)
+        self.refresh_timer_lock = threading.RLock()
+        self.refresh_timer: Optional[threading.Timer] = None
 
-    @async_tasking_event
-    async def collect(sender, document: TextDocument, range: Range) -> Optional[List[InlayHint]]:  # NOSONAR
+    @event
+    def collect(sender, document: TextDocument, range: Range) -> Optional[List[InlayHint]]:
         ...
 
-    @async_tasking_event
-    async def resolve(sender, hint: InlayHint) -> Optional[InlayHint]:  # NOSONAR
+    @event
+    def resolve(sender, hint: InlayHint) -> Optional[InlayHint]:
         ...
 
     def extend_capabilities(self, capabilities: ServerCapabilities) -> None:
@@ -45,9 +45,8 @@ class InlayHintProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
             else:
                 capabilities.inlay_hint_provider = InlayHintOptions()
 
-    @rpc_method(name="textDocument/inlayHint", param_type=InlayHintParams)
-    @threaded()
-    async def _text_document_inlay_hint(
+    @rpc_method(name="textDocument/inlayHint", param_type=InlayHintParams, threaded=True)
+    def _text_document_inlay_hint(
         self,
         text_document: TextDocumentIdentifier,
         range: Range,
@@ -60,7 +59,7 @@ class InlayHintProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
         if document is None:
             return None
 
-        for result in await self.collect(
+        for result in self.collect(
             self,
             document,
             document.range_from_utf16(range),
@@ -74,23 +73,17 @@ class InlayHintProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
                     results.extend(result)
 
         if results:
-            for result in results:
-                result.position = document.position_to_utf16(result.position)
+            for r in results:
+                r.position = document.position_to_utf16(r.position)
                 # TODO: resolve
 
             return results
 
         return None
 
-    @rpc_method(name="inlayHint/resolve", param_type=InlayHint)
-    @threaded()
-    async def _inlay_hint_resolve(
-        self,
-        params: InlayHint,
-        *args: Any,
-        **kwargs: Any,
-    ) -> Optional[InlayHint]:
-        for result in await self.resolve(self, params):
+    @rpc_method(name="inlayHint/resolve", param_type=InlayHint, threaded=True)
+    def _inlay_hint_resolve(self, params: InlayHint, *args: Any, **kwargs: Any) -> Optional[InlayHint]:
+        for result in self.resolve(self, params):
             if isinstance(result, BaseException):
                 if not isinstance(result, CancelledError):
                     self._logger.exception(result, exc_info=result)
@@ -100,11 +93,27 @@ class InlayHintProtocolPart(LanguageServerProtocolPart, HasExtendCapabilities):
 
         return params
 
-    async def refresh(self) -> None:
+    def refresh(self, now: bool = False) -> None:
+        with self.refresh_timer_lock:
+            if self.refresh_timer is not None:
+                self.refresh_timer.cancel()
+                self.refresh_timer = None
+
+            if not now:
+                self.refresh_timer = threading.Timer(1, self._refresh)
+                self.refresh_timer.start()
+                return
+
+        self._refresh()
+
+    def _refresh(self) -> None:
+        with self.refresh_timer_lock:
+            self.refresh_timer = None
+
         if (
             self.parent.client_capabilities is not None
             and self.parent.client_capabilities.workspace is not None
             and self.parent.client_capabilities.workspace.inlay_hint is not None
             and self.parent.client_capabilities.workspace.inlay_hint.refresh_support
         ):
-            await self.parent.send_request_async("workspace/inlayHint/refresh")
+            self.parent.send_request("workspace/inlayHint/refresh")
