@@ -1,15 +1,11 @@
-from __future__ import annotations
-
-import asyncio
 import threading
 import uuid
 import weakref
-from concurrent.futures import Future
-from dataclasses import dataclass
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
+    ClassVar,
     Coroutine,
     Dict,
     Final,
@@ -17,20 +13,15 @@ from typing import (
     Mapping,
     NamedTuple,
     Optional,
-    Protocol,
     Tuple,
     Type,
     TypeVar,
     Union,
     cast,
-    runtime_checkable,
 )
 
-from robotcode.core.async_tools import (
-    Lock,
-    async_tasking_event,
-    threaded,
-)
+from robotcode.core.concurrent import Task
+from robotcode.core.event import event
 from robotcode.core.lsp.types import (
     ApplyWorkspaceEditParams,
     ApplyWorkspaceEditResult,
@@ -62,17 +53,12 @@ from robotcode.core.lsp.types import (
     WorkspaceFoldersChangeEvent,
     WorkspaceFoldersServerCapabilities,
 )
-from robotcode.core.lsp.types import (
-    WorkspaceFolder as TypesWorkspaceFolder,
-)
+from robotcode.core.lsp.types import WorkspaceFolder as TypesWorkspaceFolder
 from robotcode.core.uri import Uri
 from robotcode.core.utils.dataclasses import CamelSnakeMixin, from_dict
 from robotcode.core.utils.logging import LoggingDescriptor
 from robotcode.core.utils.path import path_is_relative_to
 from robotcode.jsonrpc2.protocol import rpc_method
-from robotcode.language_server.common.has_extend_capabilities import (
-    HasExtendCapabilities,
-)
 
 from .protocol_part import LanguageServerProtocolPart
 
@@ -109,12 +95,12 @@ class FileWatcherEntry:
         self.parent: Optional[FileWatcherEntry] = None
         self.finalizer: Any = None
 
-    @async_tasking_event
-    async def child_callbacks(sender, changes: List[FileEvent]) -> None:  # NOSONAR
+    @event
+    def child_callbacks(sender, changes: List[FileEvent]) -> None:
         ...
 
-    async def call_childrens(self, sender: Any, changes: List[FileEvent]) -> None:
-        await self.child_callbacks(sender, changes)
+    def call_childrens(self, sender: Any, changes: List[FileEvent]) -> None:
+        self.child_callbacks(sender, changes)
 
     def __str__(self) -> str:
         return self.id
@@ -131,6 +117,9 @@ class WorkspaceFolder:
         self.document_uri = document_uri
 
 
+_F = TypeVar("_F", bound=Callable[..., Any])
+
+
 def config_section(name: str) -> Callable[[_F], _F]:
     def decorator(func: _F) -> _F:
         setattr(func, "__config_section__", name)
@@ -139,26 +128,20 @@ def config_section(name: str) -> Callable[[_F], _F]:
     return decorator
 
 
-@runtime_checkable
-class HasConfigSection(Protocol):
-    __config_section__: str
-
-
-@dataclass
+# @dataclass
 class ConfigBase(CamelSnakeMixin):
-    pass
+    __config_section__: ClassVar[str]
 
 
 _TConfig = TypeVar("_TConfig", bound=ConfigBase)
-_F = TypeVar("_F", bound=Callable[..., Any])
 
 
-class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
+class Workspace(LanguageServerProtocolPart):
     _logger: Final = LoggingDescriptor()
 
     def __init__(
         self,
-        parent: LanguageServerProtocol,
+        parent: "LanguageServerProtocol",
         root_uri: Optional[str],
         root_path: Optional[str],
         workspace_folders: Optional[List[TypesWorkspaceFolder]] = None,
@@ -167,7 +150,7 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
         self.root_uri = root_uri
 
         self.root_path = root_path
-        self._workspace_folders_lock = Lock()
+        self._workspace_folders_lock = threading.RLock()
         self._workspace_folders: List[WorkspaceFolder] = (
             [WorkspaceFolder(w.name, Uri(w.uri), w.uri) for w in workspace_folders]
             if workspace_folders is not None
@@ -179,10 +162,21 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
         self._file_watchers_lock = threading.RLock()
 
         self.parent.on_shutdown.add(self.server_shutdown)
+        self.parent.on_initialize.add(self.server_initialize)
+        self._settings_cache: Dict[Tuple[Optional[WorkspaceFolder], str], ConfigBase] = {}
+
+    def server_initialize(self, sender: Any, initialization_options: Optional[Any] = None) -> None:
+        if (
+            initialization_options is not None
+            and isinstance(initialization_options, dict)
+            and "settings" in initialization_options
+        ):
+            self.settings = initialization_options["settings"]
 
     @property
     def workspace_folders(self) -> List[WorkspaceFolder]:
-        return self._workspace_folders
+        with self._workspace_folders_lock:
+            return self._workspace_folders
 
     @_logger.call
     def server_shutdown(self, sender: Any) -> None:
@@ -248,46 +242,45 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
     def settings(self, value: Dict[str, Any]) -> None:
         self._settings = value
 
-    @async_tasking_event
-    async def did_change_configuration(sender, settings: Dict[str, Any]) -> None:  # NOSONAR
+    @event
+    def did_change_configuration(sender, settings: Dict[str, Any]) -> None:
         ...
 
     @rpc_method(name="workspace/didChangeConfiguration", param_type=DidChangeConfigurationParams)
-    @threaded()
-    async def _workspace_did_change_configuration(self, settings: Dict[str, Any], *args: Any, **kwargs: Any) -> None:
+    def _workspace_did_change_configuration(self, settings: Dict[str, Any], *args: Any, **kwargs: Any) -> None:
         self.settings = settings
-        await self.did_change_configuration(self, settings)
+        self._settings_cache.clear()
+        self.did_change_configuration(self, settings)
 
-    @async_tasking_event
-    async def will_create_files(sender, files: List[str]) -> Optional[Mapping[str, List[TextEdit]]]:  # NOSONAR
+    @event
+    def will_create_files(sender, files: List[str]) -> Optional[Mapping[str, List[TextEdit]]]:
         ...
 
-    @async_tasking_event
-    async def did_create_files(sender, files: List[str]) -> None:  # NOSONAR
+    @event
+    def did_create_files(sender, files: List[str]) -> None:
         ...
 
-    @async_tasking_event
-    async def will_rename_files(sender, files: List[Tuple[str, str]]) -> None:  # NOSONAR
+    @event
+    def will_rename_files(sender, files: List[Tuple[str, str]]) -> None:
         ...
 
-    @async_tasking_event
-    async def did_rename_files(sender, files: List[Tuple[str, str]]) -> None:  # NOSONAR
+    @event
+    def did_rename_files(sender, files: List[Tuple[str, str]]) -> None:
         ...
 
-    @async_tasking_event
-    async def will_delete_files(sender, files: List[str]) -> None:  # NOSONAR
+    @event
+    def will_delete_files(sender, files: List[str]) -> None:
         ...
 
-    @async_tasking_event
-    async def did_delete_files(sender, files: List[str]) -> None:  # NOSONAR
+    @event
+    def did_delete_files(sender, files: List[str]) -> None:
         ...
 
-    @rpc_method(name="workspace/willCreateFiles", param_type=CreateFilesParams)
-    @threaded()
-    async def _workspace_will_create_files(
+    @rpc_method(name="workspace/willCreateFiles", param_type=CreateFilesParams, threaded=True)
+    def _workspace_will_create_files(
         self, files: List[FileCreate], *args: Any, **kwargs: Any
     ) -> Optional[WorkspaceEdit]:
-        results = await self.will_create_files(self, [f.uri for f in files])
+        results = self.will_create_files(self, [f.uri for f in files])
         if len(results) == 0:
             return None
 
@@ -300,42 +293,29 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
 
         return WorkspaceEdit(changes=result)
 
-    @rpc_method(name="workspace/didCreateFiles", param_type=CreateFilesParams)
-    @threaded()
-    async def _workspace_did_create_files(self, files: List[FileCreate], *args: Any, **kwargs: Any) -> None:
-        await self.did_create_files(self, [f.uri for f in files])
+    @rpc_method(name="workspace/didCreateFiles", param_type=CreateFilesParams, threaded=True)
+    def _workspace_did_create_files(self, files: List[FileCreate], *args: Any, **kwargs: Any) -> None:
+        self.did_create_files(self, [f.uri for f in files])
 
-    @rpc_method(name="workspace/willRenameFiles", param_type=RenameFilesParams)
-    @threaded()
-    async def _workspace_will_rename_files(self, files: List[FileRename], *args: Any, **kwargs: Any) -> None:
-        await self.will_rename_files(self, [(f.old_uri, f.new_uri) for f in files])
+    @rpc_method(name="workspace/willRenameFiles", param_type=RenameFilesParams, threaded=True)
+    def _workspace_will_rename_files(self, files: List[FileRename], *args: Any, **kwargs: Any) -> None:
+        self.will_rename_files(self, [(f.old_uri, f.new_uri) for f in files])
 
         # TODO: return WorkspaceEdit
 
-    @rpc_method(name="workspace/didRenameFiles", param_type=RenameFilesParams)
-    @threaded()
-    async def _workspace_did_rename_files(self, files: List[FileRename], *args: Any, **kwargs: Any) -> None:
-        await self.did_rename_files(self, [(f.old_uri, f.new_uri) for f in files])
+    @rpc_method(name="workspace/didRenameFiles", param_type=RenameFilesParams, threaded=True)
+    def _workspace_did_rename_files(self, files: List[FileRename], *args: Any, **kwargs: Any) -> None:
+        self.did_rename_files(self, [(f.old_uri, f.new_uri) for f in files])
 
-    @rpc_method(name="workspace/willDeleteFiles", param_type=DeleteFilesParams)
-    @threaded()
-    async def _workspace_will_delete_files(self, files: List[FileDelete], *args: Any, **kwargs: Any) -> None:
-        await self.will_delete_files(self, [f.uri for f in files])
+    @rpc_method(name="workspace/willDeleteFiles", param_type=DeleteFilesParams, threaded=True)
+    def _workspace_will_delete_files(self, files: List[FileDelete], *args: Any, **kwargs: Any) -> None:
+        self.will_delete_files(self, [f.uri for f in files])
 
         # TODO: return WorkspaceEdit
 
-    @rpc_method(name="workspace/didDeleteFiles", param_type=DeleteFilesParams)
-    @threaded()
-    async def _workspace_did_delete_files(self, files: List[FileDelete], *args: Any, **kwargs: Any) -> None:
-        await self.did_delete_files(self, [f.uri for f in files])
-
-    def get_configuration_async(
-        self,
-        section: Type[_TConfig],
-        scope_uri: Union[str, Uri, None] = None,
-        request: bool = True,
-    ) -> asyncio.Future[_TConfig]:
-        return asyncio.wrap_future(self.get_configuration_future(section, scope_uri, request))
+    @rpc_method(name="workspace/didDeleteFiles", param_type=DeleteFilesParams, threaded=True)
+    def _workspace_did_delete_files(self, files: List[FileDelete], *args: Any, **kwargs: Any) -> None:
+        self.did_delete_files(self, [f.uri for f in files])
 
     def get_configuration(
         self,
@@ -350,10 +330,21 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
         section: Type[_TConfig],
         scope_uri: Union[str, Uri, None] = None,
         request: bool = True,
-    ) -> Future[_TConfig]:
-        result_future: Future[_TConfig] = Future()
+    ) -> Task[_TConfig]:
+        result_future: Task[_TConfig] = Task()
 
-        def _get_configuration_done(f: Future[Optional[Any]]) -> None:
+        scope = self.get_workspace_folder(scope_uri) if scope_uri is not None else None
+
+        if (scope, section.__config_section__) in self._settings_cache:
+            result_future.set_result(
+                cast(
+                    _TConfig,
+                    self._settings_cache[(scope, section.__config_section__)],
+                )
+            )
+            return result_future
+
+        def _get_configuration_done(f: Task[Optional[Any]]) -> None:
             try:
                 if result_future.cancelled():
                     return
@@ -367,12 +358,14 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
                     return
 
                 result = f.result()
-                result_future.set_result(from_dict(result[0] if result else {}, section))
+                r = from_dict(result[0] if result else {}, section)
+                self._settings_cache[(scope, section.__config_section__)] = r
+                result_future.set_result(r)
             except Exception as e:
                 result_future.set_exception(e)
 
         self.get_configuration_raw(
-            section=cast(HasConfigSection, section).__config_section__,
+            section=section.__config_section__,
             scope_uri=scope_uri,
             request=request,
         ).add_done_callback(_get_configuration_done)
@@ -384,7 +377,7 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
         section: Optional[str],
         scope_uri: Union[str, Uri, None] = None,
         request: bool = True,
-    ) -> Future[Optional[Any]]:
+    ) -> Task[Optional[Any]]:
         if (
             self.parent.client_capabilities
             and self.parent.client_capabilities.workspace
@@ -411,7 +404,7 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
             else:
                 result = {}
                 break
-        result_future: Future[Optional[Any]] = Future()
+        result_future: Task[Optional[Any]] = Task()
         result_future.set_result([result])
         return result_future
 
@@ -430,15 +423,11 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
 
         return None
 
-    @rpc_method(
-        name="workspace/didChangeWorkspaceFolders",
-        param_type=DidChangeWorkspaceFoldersParams,
-    )
-    @threaded()
-    async def _workspace_did_change_workspace_folders(
+    @rpc_method(name="workspace/didChangeWorkspaceFolders", param_type=DidChangeWorkspaceFoldersParams)
+    def _workspace_did_change_workspace_folders(
         self, event: WorkspaceFoldersChangeEvent, *args: Any, **kwargs: Any
     ) -> None:
-        async with self._workspace_folders_lock:
+        with self._workspace_folders_lock:
             to_remove: List[WorkspaceFolder] = []
             for removed in event.removed:
                 to_remove += [w for w in self._workspace_folders if w.uri == removed.uri]
@@ -449,19 +438,24 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
             for r in to_remove:
                 self._workspace_folders.remove(r)
 
+                settings_to_remove = [k for k in self._settings_cache.keys() if k[0] == r]
+                for k in settings_to_remove:
+                    self._settings_cache.pop(k, None)
+
             for a in event.added:
                 self._workspace_folders.append(WorkspaceFolder(a.name, Uri(a.uri), a.uri))
 
-    @async_tasking_event
-    async def did_change_watched_files(sender, changes: List[FileEvent]) -> None:  # NOSONAR
+        # TODO: do we need an event for this?
+
+    @event
+    def did_change_watched_files(sender, changes: List[FileEvent]) -> None:
         ...
 
-    @rpc_method(name="workspace/didChangeWatchedFiles", param_type=DidChangeWatchedFilesParams)
-    @threaded()
-    async def _workspace_did_change_watched_files(self, changes: List[FileEvent], *args: Any, **kwargs: Any) -> None:
+    @rpc_method(name="workspace/didChangeWatchedFiles", param_type=DidChangeWatchedFilesParams, threaded=True)
+    def _workspace_did_change_watched_files(self, changes: List[FileEvent], *args: Any, **kwargs: Any) -> None:
         changes = [e for e in changes if not e.uri.endswith("/globalStorage")]
         if changes:
-            await self.did_change_watched_files(self, changes)
+            self.did_change_watched_files(self, changes)
 
     def add_file_watcher(
         self,
@@ -491,7 +485,10 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
 
             entry = FileWatcherEntry(id=str(uuid.uuid4()), callback=callback, watchers=_watchers)
 
-            current_entry = next((e for e in self._file_watchers if e.watchers == _watchers), None)
+            current_entry = next(
+                (e for e in self._file_watchers if e.watchers == _watchers),
+                None,
+            )
 
             if current_entry is not None:
                 if callback not in self.did_change_watched_files:
@@ -511,7 +508,7 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
                     and self.parent.client_capabilities.workspace.did_change_watched_files.dynamic_registration
                 ):
 
-                    def _done(f: Future[None]) -> None:
+                    def _done(f: Task[None]) -> None:
                         if f.cancelled():
                             return
                         exception = f.exception()
@@ -563,7 +560,9 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
                     self.parent.unregister_capability(entry.id, "workspace/didChangeWatchedFiles")
                 # TODO: implement own filewatcher if not supported by language server client
 
-    async def apply_edit(self, edit: WorkspaceEdit, label: Optional[str] = None) -> ApplyWorkspaceEditResult:
+    def apply_edit(
+        self, edit: WorkspaceEdit, label: Optional[str] = None, timeout: Optional[float] = None
+    ) -> ApplyWorkspaceEditResult:
         if edit.changes:
             for uri, changes in edit.changes.items():
                 if changes:
@@ -578,11 +577,11 @@ class Workspace(LanguageServerProtocolPart, HasExtendCapabilities):
                     for e in doc_change.edits:
                         e.range = doc.range_to_utf16(e.range)
 
-        r = await self.parent.send_request_async(
+        r = self.parent.send_request(
             "workspace/applyEdit",
             ApplyWorkspaceEditParams(edit, label),
             return_type=ApplyWorkspaceEditResult,
-        )
+        ).result(timeout)
 
         assert r is not None
 
