@@ -7,7 +7,6 @@ from io import StringIO
 from tokenize import TokenError, generate_tokens
 from typing import (
     Any,
-    AsyncIterator,
     Iterator,
     List,
     Optional,
@@ -22,7 +21,19 @@ from robot.parsing.lexer.tokens import Token
 from robot.utils.escaping import split_from_equals, unescape
 from robot.variables.finders import NOT_FOUND, NumberFinder
 from robot.variables.search import contains_variable, search_variable
+
 from robotcode.core.lsp.types import Position
+from robotcode.robot.diagnostics.entities import (
+    LibraryEntry,
+    VariableDefinition,
+    VariableNotFoundDefinition,
+)
+from robotcode.robot.diagnostics.library_doc import (
+    ArgumentInfo,
+    KeywordArgumentKind,
+    KeywordDoc,
+    LibraryDoc,
+)
 from robotcode.robot.utils import get_robot_version
 from robotcode.robot.utils.ast import (
     iter_over_keyword_names_and_owners,
@@ -33,26 +44,12 @@ from robotcode.robot.utils.ast import (
     whitespace_from_begin_of_token,
 )
 
-from .entities import (
-    LibraryEntry,
-    VariableDefinition,
-    VariableNotFoundDefinition,
-)
-from .library_doc import (
-    ArgumentInfo,
-    KeywordArgumentKind,
-    KeywordDoc,
-    LibraryDoc,
-)
-from .namespace import (
-    DEFAULT_BDD_PREFIXES,
-    Namespace,
-)
+from .namespace import DEFAULT_BDD_PREFIXES, Namespace
 
 
-class ModelHelperMixin:
+class ModelHelper:
     @classmethod
-    async def get_run_keyword_keyworddoc_and_token_from_position(
+    def get_run_keyword_keyworddoc_and_token_from_position(
         cls,
         keyword_doc: Optional[KeywordDoc],
         argument_tokens: List[Token],
@@ -63,7 +60,7 @@ class ModelHelperMixin:
             return None, argument_tokens
 
         if keyword_doc.is_run_keyword() and len(argument_tokens) > 0:
-            result = await cls.get_keyworddoc_and_token_from_position(
+            result = cls.get_keyworddoc_and_token_from_position(
                 unescape(argument_tokens[0].value),
                 argument_tokens[0],
                 argument_tokens[1:],
@@ -75,7 +72,7 @@ class ModelHelperMixin:
         if keyword_doc.is_run_keyword_with_condition() and len(argument_tokens) > (
             cond_count := keyword_doc.run_keyword_condition_count()
         ):
-            result = await cls.get_keyworddoc_and_token_from_position(
+            result = cls.get_keyworddoc_and_token_from_position(
                 unescape(argument_tokens[cond_count].value),
                 argument_tokens[cond_count],
                 argument_tokens[cond_count + 1 :],
@@ -103,9 +100,7 @@ class ModelHelperMixin:
                     else:
                         args = []
 
-                result = await cls.get_keyworddoc_and_token_from_position(
-                    unescape(t.value), t, args, namespace, position
-                )
+                result = cls.get_keyworddoc_and_token_from_position(unescape(t.value), t, args, namespace, position)
                 if result is not None and result[0] is not None:
                     return result, []
 
@@ -126,14 +121,14 @@ class ModelHelperMixin:
                         break
                     argument_tokens = argument_tokens[1:]
 
-            inner_keyword_doc = await namespace.find_keyword(argument_tokens[1].value, raise_keyword_error=False)
+            inner_keyword_doc = namespace.find_keyword(argument_tokens[1].value, raise_keyword_error=False)
 
             if position.is_in_range(range_from_token(argument_tokens[1])):
                 return (inner_keyword_doc, argument_tokens[1]), argument_tokens[2:]
 
             argument_tokens = argument_tokens[2:]
 
-            inner_keyword_doc_and_args = await cls.get_run_keyword_keyworddoc_and_token_from_position(
+            inner_keyword_doc_and_args = cls.get_run_keyword_keyworddoc_and_token_from_position(
                 inner_keyword_doc, argument_tokens, namespace, position
             )
 
@@ -146,15 +141,21 @@ class ModelHelperMixin:
 
             while argument_tokens:
                 if argument_tokens[0].value == "ELSE" and len(argument_tokens) > 1:
-                    inner_keyword_doc = await namespace.find_keyword(unescape(argument_tokens[1].value))
+                    inner_keyword_doc = namespace.find_keyword(unescape(argument_tokens[1].value))
 
                     if position.is_in_range(range_from_token(argument_tokens[1])):
-                        return (inner_keyword_doc, argument_tokens[1]), argument_tokens[2:]
+                        return (
+                            inner_keyword_doc,
+                            argument_tokens[1],
+                        ), argument_tokens[2:]
 
                     argument_tokens = argument_tokens[2:]
 
-                    inner_keyword_doc_and_args = await cls.get_run_keyword_keyworddoc_and_token_from_position(
-                        inner_keyword_doc, argument_tokens, namespace, position
+                    inner_keyword_doc_and_args = cls.get_run_keyword_keyworddoc_and_token_from_position(
+                        inner_keyword_doc,
+                        argument_tokens,
+                        namespace,
+                        position,
                     )
 
                     if inner_keyword_doc_and_args[0] is not None:
@@ -166,15 +167,21 @@ class ModelHelperMixin:
 
                     break
                 if argument_tokens[0].value == "ELSE IF" and len(argument_tokens) > 2:
-                    inner_keyword_doc = await namespace.find_keyword(unescape(argument_tokens[2].value))
+                    inner_keyword_doc = namespace.find_keyword(unescape(argument_tokens[2].value))
 
                     if position.is_in_range(range_from_token(argument_tokens[2])):
-                        return (inner_keyword_doc, argument_tokens[2]), argument_tokens[3:]
+                        return (
+                            inner_keyword_doc,
+                            argument_tokens[2],
+                        ), argument_tokens[3:]
 
                     argument_tokens = argument_tokens[3:]
 
-                    inner_keyword_doc_and_args = await cls.get_run_keyword_keyworddoc_and_token_from_position(
-                        inner_keyword_doc, argument_tokens, namespace, position
+                    inner_keyword_doc_and_args = cls.get_run_keyword_keyworddoc_and_token_from_position(
+                        inner_keyword_doc,
+                        argument_tokens,
+                        namespace,
+                        position,
                     )
 
                     if inner_keyword_doc_and_args[0] is not None:
@@ -189,7 +196,7 @@ class ModelHelperMixin:
         return None, argument_tokens
 
     @classmethod
-    async def get_keyworddoc_and_token_from_position(
+    def get_keyworddoc_and_token_from_position(
         cls,
         keyword_name: Optional[str],
         keyword_token: Token,
@@ -198,7 +205,7 @@ class ModelHelperMixin:
         position: Position,
         analyse_run_keywords: bool = True,
     ) -> Optional[Tuple[Optional[KeywordDoc], Token]]:
-        keyword_doc = await namespace.find_keyword(keyword_name, raise_keyword_error=False)
+        keyword_doc = namespace.find_keyword(keyword_name, raise_keyword_error=False)
         if keyword_doc is None:
             return None
 
@@ -207,7 +214,7 @@ class ModelHelperMixin:
 
         if analyse_run_keywords:
             return (
-                await cls.get_run_keyword_keyworddoc_and_token_from_position(
+                cls.get_run_keyword_keyworddoc_and_token_from_position(
                     keyword_doc, argument_tokens, namespace, position
                 )
             )[0]
@@ -215,7 +222,7 @@ class ModelHelperMixin:
         return None
 
     @classmethod
-    async def get_namespace_info_from_keyword_token(
+    def get_namespace_info_from_keyword_token(
         cls, namespace: Namespace, keyword_token: Token
     ) -> Tuple[Optional[LibraryEntry], Optional[str]]:
         lib_entry: Optional[LibraryEntry] = None
@@ -223,7 +230,10 @@ class ModelHelperMixin:
 
         for lib, keyword in iter_over_keyword_names_and_owners(keyword_token.value):
             if lib is not None:
-                lib_entries = next((v for k, v in (await namespace.get_namespaces()).items() if k == lib), None)
+                lib_entries = next(
+                    (v for k, v in (namespace.get_namespaces()).items() if k == lib),
+                    None,
+                )
                 if lib_entries is not None:
                     kw_namespace = lib
                     lib_entry = next(
@@ -243,20 +253,20 @@ class ModelHelperMixin:
     )
 
     @staticmethod
-    async def iter_expression_variables_from_token(
+    def iter_expression_variables_from_token(
         expression: Token,
         namespace: Namespace,
         nodes: Optional[List[ast.AST]],
         position: Optional[Position] = None,
         skip_commandline_variables: bool = False,
         return_not_found: bool = False,
-    ) -> AsyncIterator[Tuple[Token, VariableDefinition]]:
+    ) -> Iterator[Tuple[Token, VariableDefinition]]:
         variable_started = False
         try:
             for toknum, tokval, (_, tokcol), _, _ in generate_tokens(StringIO(expression.value).readline):
                 if variable_started:
                     if toknum == python_token.NAME:
-                        var = await namespace.find_variable(
+                        var = namespace.find_variable(
                             f"${{{tokval}}}",
                             nodes,
                             position,
@@ -273,14 +283,17 @@ class ModelHelperMixin:
                         if var is not None:
                             yield sub_token, var
                         elif return_not_found:
-                            yield sub_token, VariableNotFoundDefinition(
-                                sub_token.lineno,
-                                sub_token.col_offset,
-                                sub_token.lineno,
-                                sub_token.end_col_offset,
-                                namespace.source,
-                                tokval,
+                            yield (
                                 sub_token,
+                                VariableNotFoundDefinition(
+                                    sub_token.lineno,
+                                    sub_token.col_offset,
+                                    sub_token.lineno,
+                                    sub_token.end_col_offset,
+                                    namespace.source,
+                                    tokval,
+                                    sub_token,
+                                ),
                             )
                     variable_started = False
                 if tokval == "$":
@@ -289,7 +302,9 @@ class ModelHelperMixin:
             pass
 
     @staticmethod
-    def remove_index_from_variable_token(token: Token) -> Tuple[Token, Optional[Token]]:
+    def remove_index_from_variable_token(
+        token: Token,
+    ) -> Tuple[Token, Optional[Token]]:
         def escaped(i: int) -> bool:
             return bool(token.value[-i - 3 : -i - 2] == "\\")
 
@@ -331,7 +346,7 @@ class ModelHelperMixin:
         return (var, rest)
 
     @classmethod
-    def _tokenize_variables(
+    def tokenize_variables(
         cls,
         token: Token,
         identifiers: str = "$@&%",
@@ -345,12 +360,17 @@ class ModelHelperMixin:
                 if var is not None:
                     yield var
                 if rest is not None:
-                    yield from cls._tokenize_variables(rest, identifiers, ignore_errors, extra_types=extra_types)
+                    yield from cls.tokenize_variables(
+                        rest,
+                        identifiers,
+                        ignore_errors,
+                        extra_types=extra_types,
+                    )
             else:
                 yield t
 
     @classmethod
-    async def iter_variables_from_token(
+    def iter_variables_from_token(
         cls,
         token: Token,
         namespace: Namespace,
@@ -358,23 +378,23 @@ class ModelHelperMixin:
         position: Optional[Position] = None,
         skip_commandline_variables: bool = False,
         return_not_found: bool = False,
-    ) -> AsyncIterator[Tuple[Token, VariableDefinition]]:
+    ) -> Iterator[Tuple[Token, VariableDefinition]]:
         def is_number(name: str) -> bool:
             if name.startswith("$"):
                 finder = NumberFinder()
                 return bool(finder.find(name) != NOT_FOUND)
             return False
 
-        async def iter_token(
+        def iter_token(
             to: Token, ignore_errors: bool = False
-        ) -> AsyncIterator[Union[Token, Tuple[Token, VariableDefinition]]]:
-            for sub_token in cls._tokenize_variables(to, ignore_errors=ignore_errors):
+        ) -> Iterator[Union[Token, Tuple[Token, VariableDefinition]]]:
+            for sub_token in cls.tokenize_variables(to, ignore_errors=ignore_errors):
                 if sub_token.type == Token.VARIABLE:
                     base = sub_token.value[2:-1]
                     if base and not (base[0] == "{" and base[-1] == "}"):
                         yield sub_token
                     elif base:
-                        async for v in cls.iter_expression_variables_from_token(
+                        for v in cls.iter_expression_variables_from_token(
                             Token(
                                 sub_token.type,
                                 base[1:-1],
@@ -390,19 +410,22 @@ class ModelHelperMixin:
                         ):
                             yield v
                     elif base == "" and return_not_found:
-                        yield sub_token, VariableNotFoundDefinition(
-                            sub_token.lineno,
-                            sub_token.col_offset,
-                            sub_token.lineno,
-                            sub_token.end_col_offset,
-                            namespace.source,
-                            sub_token.value,
+                        yield (
                             sub_token,
+                            VariableNotFoundDefinition(
+                                sub_token.lineno,
+                                sub_token.col_offset,
+                                sub_token.lineno,
+                                sub_token.end_col_offset,
+                                namespace.source,
+                                sub_token.value,
+                                sub_token,
+                            ),
                         )
                         return
 
                     if contains_variable(base, "$@&%"):
-                        async for sub_token_or_var in iter_token(
+                        for sub_token_or_var in iter_token(
                             Token(
                                 to.type,
                                 base,
@@ -430,11 +453,11 @@ class ModelHelperMixin:
                 token.error,
             )
 
-        async for token_or_var in iter_token(token, ignore_errors=True):
+        for token_or_var in iter_token(token, ignore_errors=True):
             if isinstance(token_or_var, Token):
                 sub_token = token_or_var
                 name = sub_token.value
-                var = await namespace.find_variable(
+                var = namespace.find_variable(
                     name,
                     nodes,
                     position,
@@ -458,14 +481,19 @@ class ModelHelperMixin:
                     if match is not None:
                         base_name, _ = match.groups()
                         name = f"{name[0]}{{{base_name.strip()}}}"
-                        var = await namespace.find_variable(
+                        var = namespace.find_variable(
                             name,
                             nodes,
                             position,
                             skip_commandline_variables=skip_commandline_variables,
                             ignore_error=True,
                         )
-                        sub_sub_token = Token(sub_token.type, name, sub_token.lineno, sub_token.col_offset)
+                        sub_sub_token = Token(
+                            sub_token.type,
+                            name,
+                            sub_token.lineno,
+                            sub_token.col_offset,
+                        )
                         if var is not None:
                             yield strip_variable_token(sub_sub_token), var
                             continue
@@ -475,24 +503,30 @@ class ModelHelperMixin:
                             if contains_variable(sub_token.value[2:-1]):
                                 continue
                             else:
-                                yield strip_variable_token(sub_sub_token), VariableNotFoundDefinition(
-                                    sub_sub_token.lineno,
-                                    sub_sub_token.col_offset,
-                                    sub_sub_token.lineno,
-                                    sub_sub_token.end_col_offset,
-                                    namespace.source,
-                                    name,
-                                    sub_sub_token,
+                                yield (
+                                    strip_variable_token(sub_sub_token),
+                                    VariableNotFoundDefinition(
+                                        sub_sub_token.lineno,
+                                        sub_sub_token.col_offset,
+                                        sub_sub_token.lineno,
+                                        sub_sub_token.end_col_offset,
+                                        namespace.source,
+                                        name,
+                                        sub_sub_token,
+                                    ),
                                 )
                 if return_not_found:
-                    yield strip_variable_token(sub_token), VariableNotFoundDefinition(
-                        sub_token.lineno,
-                        sub_token.col_offset,
-                        sub_token.lineno,
-                        sub_token.end_col_offset,
-                        namespace.source,
-                        sub_token.value,
-                        sub_token,
+                    yield (
+                        strip_variable_token(sub_token),
+                        VariableNotFoundDefinition(
+                            sub_token.lineno,
+                            sub_token.col_offset,
+                            sub_token.lineno,
+                            sub_token.end_col_offset,
+                            namespace.source,
+                            sub_token.value,
+                            sub_token,
+                        ),
                     )
             else:
                 yield token_or_var
@@ -737,7 +771,11 @@ class ModelHelperMixin:
                 (
                     i
                     for i, v in enumerate(kw_arguments)
-                    if v.kind in [KeywordArgumentKind.POSITIONAL_ONLY, KeywordArgumentKind.POSITIONAL_OR_NAMED]
+                    if v.kind
+                    in [
+                        KeywordArgumentKind.POSITIONAL_ONLY,
+                        KeywordArgumentKind.POSITIONAL_OR_NAMED,
+                    ]
                     and i == argument_index
                 ),
                 -1,
@@ -747,11 +785,13 @@ class ModelHelperMixin:
             else:
                 if need_named:
                     argument_index = next(
-                        (i for i, v in enumerate(kw_arguments) if v.kind == KeywordArgumentKind.VAR_NAMED), -1
+                        (i for i, v in enumerate(kw_arguments) if v.kind == KeywordArgumentKind.VAR_NAMED),
+                        -1,
                     )
                 else:
                     argument_index = next(
-                        (i for i, v in enumerate(kw_arguments) if v.kind == KeywordArgumentKind.VAR_POSITIONAL), -1
+                        (i for i, v in enumerate(kw_arguments) if v.kind == KeywordArgumentKind.VAR_POSITIONAL),
+                        -1,
                     )
 
         if argument_index >= len(kw_arguments):
